"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FailedValidationError = exports.Validator = void 0;
const interfaces_1 = require("./interfaces");
Object.defineProperty(exports, "FailedValidationError", { enumerable: true, get: function () { return interfaces_1.FailedValidationError; } });
Object.defineProperty(exports, "Validator", { enumerable: true, get: function () { return interfaces_1.Validator; } });
const spread_args_to_array_1 = require("./util/spread-args-to-array");
const any_validator_1 = require("./validators/any-validator");
const array_validator_1 = require("./validators/array-validator");
const constant_validator_1 = require("./validators/constant-validator");
const number_validator_1 = require("./validators/number-validator");
const object_validator_1 = require("./validators/object-validator");
const or_validator_1 = require("./validators/or-validator");
const string_validator_1 = require("./validators/string-validator");
const tuple_validator_1 = require("./validators/tuple-validator");
const boolean_validator_1 = require("./validators/boolean-validator");
const json_validator_1 = require("./validators/json-validator");
/**
 * The default export of the jointz library that exposes static methods for constructing validators.
 */
class jointz {
    /**
     * Create a validator that checks the value is a string.
     */
    static string() {
        return new string_validator_1.default({});
    }
    /**
     * Create a validator that checks that the value is an object with a given shape.
     *
     * By default the object is not allowed to have keys other than what are specified.
     * Call `allowUnknownKeys(true)` on the returned object validator to allow the object to have keys that are not
     * specified which are not validated.
     *
     * @param keys validation to apply to each key in the object
     */
    static object(keys) {
        return new object_validator_1.default({
            keys,
            requiredKeys: [],
            allowUnknownKeys: false,
        });
    }
    /**
     * Create a validator that checks that the value is a number. Call additional methods to add constraints to the number.
     */
    static number() {
        return new number_validator_1.default({});
    }
    /**
     * Create a validator that checks that the given value is a tuple of values that match the given validators.
     * @param validators list of validators that the items in the tuple should match in order
     */
    static tuple(...validators) {
        return new tuple_validator_1.default({
            validators: (0, spread_args_to_array_1.spreadArgsToArray)(validators),
        });
    }
    /**
     * Combine a list of validators to produce a new validator that passes if any of the given validators pass
     * @param validators validators to combine into an OR expression validator
     */
    static or(...validators) {
        return new or_validator_1.default({ validators: (0, spread_args_to_array_1.spreadArgsToArray)(validators) });
    }
    /**
     * Return a validator that checks the value is an array, optionally validating each item in the array
     * @param itemValidator validator for the individual array items
     */
    static array(itemValidator) {
        return new array_validator_1.default({ items: itemValidator });
    }
    /**
     * Return a constant validator that checks that the value is one of a set of given values.
     * @param allowedValues the values that are considered valid. can be numbers, strings, booleans, `null`, or `undefined`
     */
    static constant(...allowedValues) {
        return new constant_validator_1.default({ allowedValues: allowedValues });
    }
    /**
     * Return a validator that checks that the value is either true or false.
     * This is an alias for constructing a constant validator.
     */
    static boolean() {
        return boolean_validator_1.BOOLEAN_VALIDATOR;
    }
    /**
     * Return a validator that always succeeds but does not limit the type.
     */
    static any() {
        return any_validator_1.ANY_VALIDATOR;
    }
    /**
     * Return a validator that checks the string contains JSON that when parsed matches some validation
     * @param parsed the validation to be applied to the parsed JSON
     */
    static json(parsed) {
        return new json_validator_1.default(parsed);
    }
}
exports.default = jointz;
//# sourceMappingURL=index.js.map