export declare type ValidationErrorPath = Readonly<Array<string | number>>;
/**
 * Information about a failed validation.
 */
export interface ValidationError {
    readonly path: ValidationErrorPath;
    readonly message: string;
    readonly value?: unknown;
}
/**
 * The validation error that is thrown from checkValid.
 */
export declare class FailedValidationError extends Error {
    readonly isFailedValidationError: true;
    readonly errors: ValidationError[];
    constructor(errors: ValidationError[]);
}
/**
 * The base class of all the validators in this package. TValid is the type of any value that passes validation.
 */
export declare abstract class Validator<TValid> {
    /**
     * Validate a given value with the assumed path, returning any errors
     * @param value value to validate
     * @param path current path of the validation
     */
    abstract validate(value: unknown, path?: ValidationErrorPath): ValidationError[];
    /**
     * Return true if the value is valid. #isValid is unique in that validation errors are not surfaced, so the validator
     * may optimize for performance.
     * @param value value to check
     */
    isValid(value: unknown): value is TValid;
    /**
     * Validate the given value and throw if it's not valid
     * @param value that is TValid
     * @param path The path at which the value being checked was found
     */
    checkValid(value: unknown, path?: ValidationErrorPath): TValid;
}
/**
 * Extracts the result type from a validator. Define your validator and then use this to get the type of result.
 */
export declare type Infer<TValidator> = TValidator extends Validator<infer T> ? T : TValidator extends Validator<any>[] ? {
    [P in keyof TValidator]: Infer<TValidator[P]>;
} : TValidator extends {} ? {
    [P in keyof TValidator]: Infer<TValidator[P]>;
} : unknown;
export declare type ExtractResultType<TValidator> = Infer<TValidator>;
