"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const interfaces_1 = require("../interfaces");
/**
 * Validator that checks that the value is an array of a particular kind of item
 */
class ArrayValidator extends interfaces_1.Validator {
    constructor(options) {
        super();
        this.options = options;
    }
    /**
     * A valid array must have at least min elements
     * @param min min array length
     */
    minLength(min) {
        if (min < 0) {
            throw new Error(`min length ${min} must be greater than or equal to 0`);
        }
        return new ArrayValidator(Object.assign(Object.assign({}, this.options), { minLength: min }));
    }
    /**
     * A valid array must have at most max elements
     * @param max max array length
     */
    maxLength(max) {
        if (max < 0) {
            throw new Error(`max length ${max} must be greater than or equal to 0`);
        }
        return new ArrayValidator(Object.assign(Object.assign({}, this.options), { maxLength: max }));
    }
    /**
     * Specify the type of the item allowed for the array validator
     * @param items type of items allowed in the array
     */
    items(items) {
        return new ArrayValidator(Object.assign(Object.assign({}, this.options), { items }));
    }
    validate(value, path = []) {
        const { items, minLength, maxLength } = this.options;
        if (typeof value !== "object" || !Array.isArray(value)) {
            return [{ message: `must be an array`, path, value }];
        }
        let errors = [];
        if (minLength !== undefined && value.length < minLength) {
            errors.push({
                message: `array length ${value.length} was less than minimum length: ${minLength}`,
                path,
                value,
            });
        }
        if (maxLength !== undefined && value.length > maxLength) {
            errors.push({
                message: `array length ${value.length} was greater than maximum length: ${maxLength}`,
                path,
                value,
            });
        }
        if (errors.length === 0 && items) {
            for (let i = 0; i < value.length; i++) {
                errors = errors.concat(items.validate(value[i], [...path, i]));
            }
        }
        return errors;
    }
    isValid(value) {
        const { minLength, maxLength, items } = this.options;
        if (typeof value === "object" &&
            Array.isArray(value) &&
            (minLength === undefined || value.length >= minLength) &&
            (maxLength === undefined || value.length <= maxLength)) {
            if (items) {
                for (let x of value) {
                    if (!items.isValid(x)) {
                        return false;
                    }
                }
                return true;
            }
            else {
                return true;
            }
        }
        return false;
    }
}
exports.default = ArrayValidator;
//# sourceMappingURL=array-validator.js.map