"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const interfaces_1 = require("../interfaces");
const SUPPORTED_VALUE_TYPEOF = [
    "string",
    "number",
    "boolean",
    "undefined",
];
/**
 * Return true if the given value is one of the supported value types
 * @param value to check
 */
function isSupportedValueType(value) {
    return value === null || SUPPORTED_VALUE_TYPEOF.indexOf(typeof value) !== -1;
}
/**
 * Validator that checks that a value is one of a given set of constants
 */
class ConstantValidator extends interfaces_1.Validator {
    constructor(options) {
        super();
        if (options.allowedValues.length === 0) {
            throw new Error("constant validators should have at least one value");
        }
        for (let value of options.allowedValues) {
            if (!isSupportedValueType(value)) {
                throw new Error("unsupported value type in constant validator, must be string, boolean, number, null or undefined");
            }
        }
        this.options = options;
    }
    validate(value, path = []) {
        const { allowedValues } = this.options;
        if (allowedValues.indexOf(value) === -1) {
            return [
                {
                    path,
                    message: `must be one of ${this.options.allowedValues
                        .map((v) => (typeof v === "string" ? `"${v}"` : `${v}`))
                        .join(", ")}`,
                    value,
                },
            ];
        }
        return [];
    }
    isValid(value) {
        return this.options.allowedValues.indexOf(value) !== -1;
    }
}
exports.default = ConstantValidator;
//# sourceMappingURL=constant-validator.js.map