"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const interfaces_1 = require("../interfaces");
class NumberValidator extends interfaces_1.Validator {
    constructor(options) {
        super();
        this.options = options;
    }
    /**
     * Limit the number to be greater than or equal to the min
     * @param min min number
     */
    min(min) {
        return new NumberValidator(Object.assign(Object.assign({}, this.options), { min }));
    }
    /**
     * Limit the number to be less than or equal to the max
     * @param max max number
     */
    max(max) {
        return new NumberValidator(Object.assign(Object.assign({}, this.options), { max }));
    }
    /**
     * Limit the number validator to only allow numbers that are a multiple of the given value
     * @param multipleOf number that a valid value must be a multiple of
     */
    multipleOf(multipleOf) {
        return new NumberValidator(Object.assign(Object.assign({}, this.options), { multipleOf }));
    }
    /**
     * Limit the number validator to only allow integers. Alias to #multipleOf(1).
     */
    integer() {
        return this.multipleOf(1);
    }
    validate(value, path = []) {
        const { multipleOf, min, max } = this.options;
        if (typeof value !== "number") {
            return [{ message: `must be a number`, path, value }];
        }
        const errors = [];
        if (multipleOf && value % multipleOf !== 0) {
            errors.push({
                message: multipleOf === 1
                    ? "number was not an integer"
                    : `number was not a multiple of ${multipleOf}`,
                path,
                value,
            });
        }
        if (typeof min !== "undefined" && value < min) {
            errors.push({
                message: `${value} must be greater than or equal to ${min}`,
                path,
                value,
            });
        }
        if (typeof max !== "undefined" && value > max) {
            errors.push({
                message: `${value} must be less than or equal to ${max}`,
                path,
                value,
            });
        }
        return errors;
    }
    isValid(value) {
        const { max, min, multipleOf } = this.options;
        return (typeof value === "number" &&
            (max === undefined || value <= max) &&
            (min === undefined || value >= min) &&
            (multipleOf === undefined || value % multipleOf === 0));
    }
}
exports.default = NumberValidator;
//# sourceMappingURL=number-validator.js.map