import { Infer, ValidationError, ValidationErrorPath, Validator } from "../interfaces";
export interface Keys {
    [key: string]: Validator<unknown>;
}
export declare type ExtractObjectType<TKeys extends Keys> = {
    [K in keyof TKeys]: Infer<TKeys[K]>;
};
interface ObjectValidatorOptions<TKeys extends Keys, TRequiredKeys extends keyof TKeys, AllowUnknownKeys extends boolean | Validator<string>> {
    readonly keys: TKeys;
    readonly requiredKeys: TRequiredKeys[];
    readonly allowUnknownKeys: AllowUnknownKeys;
}
declare type AllowUnknownKeyObject<TObject extends {}, AllowUnknownKeys extends boolean | Validator<string>> = AllowUnknownKeys extends false ? TObject : TObject & {
    [key: string]: unknown;
};
declare type WithRequiredKeys<TObject extends {}, TRequiredKeys extends keyof TObject> = {
    [K in keyof TObject]?: TObject[K];
} & {
    [K in keyof Pick<TObject, TRequiredKeys>]: TObject[K];
};
/**
 * Validator that checks a value is an object where each key has a value that matches a given validator.
 */
export default class ObjectValidator<TKeys extends Keys, TRequiredKeys extends keyof TKeys, TAllowUnknown extends boolean | Validator<string>> extends Validator<AllowUnknownKeyObject<WithRequiredKeys<ExtractObjectType<TKeys>, TRequiredKeys>, TAllowUnknown>> {
    private readonly options;
    constructor(options: ObjectValidatorOptions<TKeys, TRequiredKeys, TAllowUnknown>);
    /**
     * Specify which keys are required for the object to be valid. Replaces any of the existing required keys. By default
     * no keys are required.
     *
     * @param requiredKeys keys that must be present for the object to be valid
     */
    requiredKeys<T extends (keyof TKeys)[]>(...requiredKeys: T | [T]): ObjectValidator<TKeys, T[number], TAllowUnknown>;
    /**
     * Change the behavior of the object validator to allow or disallow keys to be present in the object that are not
     * specified. By default unknown keys are allowed.
     *
     * @param allowUnknownKeys whether to allow keys that are not specified in the object validator to be present
     */
    allowUnknownKeys<TAllow extends boolean | Validator<string>>(allowUnknownKeys: TAllow): ObjectValidator<TKeys, TRequiredKeys, TAllow>;
    validate(value: unknown, path?: ValidationErrorPath): ValidationError[];
    isValid(value: unknown): value is AllowUnknownKeyObject<WithRequiredKeys<ExtractObjectType<TKeys>, TRequiredKeys>, TAllowUnknown>;
}
export {};
