"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const interfaces_1 = require("../interfaces");
const spread_args_to_array_1 = require("../util/spread-args-to-array");
/**
 * Validator that checks a value is an object where each key has a value that matches a given validator.
 */
class ObjectValidator extends interfaces_1.Validator {
    constructor(options) {
        super();
        this.options = options;
    }
    /**
     * Specify which keys are required for the object to be valid. Replaces any of the existing required keys. By default
     * no keys are required.
     *
     * @param requiredKeys keys that must be present for the object to be valid
     */
    requiredKeys(...requiredKeys) {
        return new ObjectValidator(Object.assign(Object.assign({}, this.options), { requiredKeys: (0, spread_args_to_array_1.spreadArgsToArray)(requiredKeys) }));
    }
    /**
     * Change the behavior of the object validator to allow or disallow keys to be present in the object that are not
     * specified. By default unknown keys are allowed.
     *
     * @param allowUnknownKeys whether to allow keys that are not specified in the object validator to be present
     */
    allowUnknownKeys(allowUnknownKeys) {
        return new ObjectValidator(Object.assign(Object.assign({}, this.options), { allowUnknownKeys }));
    }
    validate(value, path = []) {
        const { requiredKeys, keys, allowUnknownKeys } = this.options;
        if (typeof value !== "object" || Array.isArray(value) || value === null) {
            return [{ message: `must be an object`, path, value }];
        }
        let errors = [];
        for (const key in value) {
            if (value.hasOwnProperty(key)) {
                if (keys && keys[key] !== undefined) {
                    errors = errors.concat(keys[key].validate(value[key], [
                        ...path,
                        key,
                    ]));
                }
                else {
                    if (!allowUnknownKeys) {
                        errors.push({
                            message: `encountered unknown key "${key}"`,
                            path,
                            value,
                        });
                    }
                    else if (typeof allowUnknownKeys !== "boolean") {
                        errors.push(...allowUnknownKeys.validate(key, path).map((error) => (Object.assign(Object.assign({}, error), { message: `key "${key}" failed validation: ${error.message}` }))));
                    }
                }
            }
        }
        if (requiredKeys) {
            for (const requiredKey of requiredKeys) {
                if (!value.hasOwnProperty(requiredKey)) {
                    errors.push({
                        message: `required key "${String(requiredKey)}" was not defined`,
                        path,
                        value,
                    });
                }
            }
        }
        return errors;
    }
    isValid(value) {
        const { allowUnknownKeys, keys, requiredKeys } = this.options;
        if (typeof value !== "object" || Array.isArray(value) || value === null) {
            return false;
        }
        for (const key in value) {
            if (value.hasOwnProperty(key)) {
                if (keys && keys[key] !== undefined) {
                    if (!keys[key].isValid(value[key])) {
                        return false;
                    }
                }
                else {
                    if (!allowUnknownKeys) {
                        return false;
                    }
                    else if (typeof allowUnknownKeys !== "boolean" &&
                        !allowUnknownKeys.isValid(key)) {
                        return false;
                    }
                }
            }
        }
        if (requiredKeys) {
            for (const requiredKey of requiredKeys) {
                if (!value.hasOwnProperty(requiredKey)) {
                    return false;
                }
            }
        }
        return true;
    }
}
exports.default = ObjectValidator;
//# sourceMappingURL=object-validator.js.map